@@ -20,6 +20,7 @@ from hashlib import md5
 from six import iteritems, text_type, string_types
 from six.moves import cPickle as pickle
 
+import docutils
 from docutils import nodes
 from docutils.io import DocTreeInput, StringOutput
 from docutils.core import Publisher
@@ -34,7 +35,7 @@ from sphinx.util.inventory import InventoryFile
 from sphinx.util.osutil import SEP, os_path, relative_uri, ensuredir, \
     movefile, copyfile
 from sphinx.util.nodes import inline_all_toctrees
-from sphinx.util.docutils import is_html5_writer_available, __version_info__
+from sphinx.util.docutils import is_html5_writer_available
 from sphinx.util.fileutil import copy_asset
 from sphinx.util.matching import patmatch, Matcher, DOTFILES
 from sphinx.config import string_classes
@@ -121,6 +122,7 @@ class CSSContainer(list):
                       RemovedInSphinx20Warning)
         for item in other:
             self.append(item)
+        return self
 
     def __add__(self, other):
         ret = CSSContainer(self)
@@ -213,10 +215,9 @@ class StandaloneHTMLBuilder(Builder):
         self.use_index = self.get_builder_config('use_index', 'html')
 
         if self.config.html_experimental_html5_writer and not html5_ready:
-            self.app.warn(' '.join((
-                'html_experimental_html5_writer is set, but current version is old.',
-                'Docutils\' version should be or newer than 0.13, but %s.',
-            )) % '.'.join(map(str, __version_info__)))
+            self.app.warn(('html_experimental_html5_writer is set, but current version '
+                           'is old. Docutils\' version should be 0.13 or newer, but %s.') %
+                          docutils.__version__)
 
     def _get_translations_js(self):
         # type: () -> unicode
@@ -260,7 +261,11 @@ class StandaloneHTMLBuilder(Builder):
 
     @property
     def default_translator_class(self):
-        if self.config.html_experimental_html5_writer and html5_ready:
+        use_html5_writer = self.config.html_experimental_html5_writer
+        if use_html5_writer is None:
+            use_html5_writer = self.default_html5_translator
+
+        if use_html5_writer and html5_ready:
             return HTML5Translator
         else:
             return HTMLTranslator
@@ -600,7 +605,7 @@ class StandaloneHTMLBuilder(Builder):
 
         # additional pages from conf.py
         for pagename, template in self.config.html_additional_pages.items():
-            self.info(' ' + pagename, nonl=1)
+            logger.info(' ' + pagename, nonl=1)
             self.handle_page(pagename, {}, template)
 
         # the search page
@@ -893,7 +898,6 @@ class StandaloneHTMLBuilder(Builder):
                     outfilename=None, event_arg=None):
         # type: (unicode, Dict, unicode, unicode, Any) -> None
         ctx = self.globalcontext.copy()
-        ctx['warn'] = self.warn
         # current_page_name is backwards compatibility
         ctx['pagename'] = ctx['current_page_name'] = pagename
         ctx['encoding'] = self.config.html_output_encoding
@@ -926,6 +930,13 @@ class StandaloneHTMLBuilder(Builder):
             return False
         ctx['hasdoc'] = hasdoc
 
+        def warn(*args, **kwargs):
+            # type: (Any, Any) -> unicode
+            """Simple warn() wrapper for themes."""
+            self.warn(*args, **kwargs)
+            return ''  # return empty string
+        ctx['warn'] = warn
+
         ctx['toctree'] = lambda **kw: self._get_local_toctree(pagename, **kw)
         self.add_sidebars(pagename, ctx)
         ctx.update(addctx)
@@ -1183,7 +1194,7 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
 
         # additional pages from conf.py
         for pagename, template in self.config.html_additional_pages.items():
-            self.info(' ' + pagename, nonl=1)
+            logger.info(' ' + pagename, nonl=1)
             self.handle_page(pagename, {}, template)
 
         if self.config.html_use_opensearch:
