@@ -12,13 +12,14 @@ from __future__ import absolute_import
 
 import sys
 import warnings
+from distutils.version import LooseVersion
 
 from docutils.parsers.rst import Directive  # noqa
 from docutils import __version__ as _du_version
 
 from sphinx.deprecation import RemovedInSphinx17Warning
 
-docutils_version = tuple(int(x) for x in _du_version.split('.')[:2])
+docutils_version = tuple(LooseVersion(_du_version).version)[:2]
 
 if False:
     # For type annotation
@@ -34,15 +35,14 @@ class _DeprecationWrapper(object):
     def __getattr__(self, attr):
         # type: (str) -> Any
         if attr in self._deprecated:
-            warnings.warn("sphinx.util.compat.%s is deprecated and will be "
-                          "removed in Sphinx 1.7, please use the standard "
-                          "library version instead." % attr,
-                          RemovedInSphinx17Warning, stacklevel=2)
+            warnings.warn("sphinx.util.compat.%s is deprecated and will be removed "
+                          "in Sphinx 1.7, please use docutils' instead." % attr,
+                          RemovedInSphinx17Warning)
             return self._deprecated[attr]
         return getattr(self._mod, attr)
 
 
-sys.modules[__name__] = _DeprecationWrapper(sys.modules[__name__], dict(  # type: ignore
+sys.modules[__name__] = _DeprecationWrapper(sys.modules[__name__], dict(
     docutils_version = docutils_version,
     Directive = Directive,
 ))
