@@ -98,15 +98,29 @@ DEFAULT_SETTINGS = {
     'tocdepth':        '',
     'secnumdepth':     '',
     'pageautorefname': '',
+    'literalblockpto': '',
 }  # type: Dict[unicode, unicode]
 
 ADDITIONAL_SETTINGS = {
     'pdflatex': {
         'inputenc':     '\\usepackage[utf8]{inputenc}',
         'utf8extra':   ('\\ifdefined\\DeclareUnicodeCharacter\n'
-                        ' \\ifdefined\\DeclareUnicodeCharacterAsOptional\\else\n'
+                        ' \\ifdefined\\DeclareUnicodeCharacterAsOptional\n'
+                        '  \\DeclareUnicodeCharacter{"00A0}{\\nobreakspace}\n'
+                        '  \\DeclareUnicodeCharacter{"2500}{\\sphinxunichar{2500}}\n'
+                        '  \\DeclareUnicodeCharacter{"2502}{\\sphinxunichar{2502}}\n'
+                        '  \\DeclareUnicodeCharacter{"2514}{\\sphinxunichar{2514}}\n'
+                        '  \\DeclareUnicodeCharacter{"251C}{\\sphinxunichar{251C}}\n'
+                        '  \\DeclareUnicodeCharacter{"2572}{\\textbackslash}\n'
+                        ' \\else\n'
                         '  \\DeclareUnicodeCharacter{00A0}{\\nobreakspace}\n'
-                        '\\fi\\fi'),
+                        '  \\DeclareUnicodeCharacter{2500}{\\sphinxunichar{2500}}\n'
+                        '  \\DeclareUnicodeCharacter{2502}{\\sphinxunichar{2502}}\n'
+                        '  \\DeclareUnicodeCharacter{2514}{\\sphinxunichar{2514}}\n'
+                        '  \\DeclareUnicodeCharacter{251C}{\\sphinxunichar{251C}}\n'
+                        '  \\DeclareUnicodeCharacter{2572}{\\textbackslash}\n'
+                        ' \\fi\n'
+                        '\\fi'),
     },
     'xelatex': {
         'latex_engine': 'xelatex',
@@ -650,6 +664,14 @@ class LaTeXTranslator(nodes.NodeVisitor):
         if self.elements['extraclassoptions']:
             self.elements['classoptions'] += ',' + \
                                              self.elements['extraclassoptions']
+        self.elements['literalblockpto'] = (
+            self.babel_renewcommand(
+                '\\literalblockcontinuedname', self.encode(_('continued from previous page'))
+            ) +
+            self.babel_renewcommand(
+                '\\literalblockcontinuesname', self.encode(_('continues on next page'))
+            )
+        )
         self.elements['pageautorefname'] = \
             self.babel_defmacro('\\pageautorefname', self.encode(_('page')))
         self.elements['numfig_format'] = self.generate_numfig_format(builder)
@@ -1222,7 +1244,7 @@ class LaTeXTranslator(nodes.NodeVisitor):
 
     def visit_desc_annotation(self, node):
         # type: (nodes.Node) -> None
-        self.body.append(r'\sphinxstrong{')
+        self.body.append(r'\sphinxbfcode{')
 
     def depart_desc_annotation(self, node):
         # type: (nodes.Node) -> None
@@ -1441,7 +1463,8 @@ class LaTeXTranslator(nodes.NodeVisitor):
         if cell.width > 1 or cell.height > 1:
             self.body.append('\\begin{varwidth}[t]{\\sphinxcolwidth{%d}{%d}}\n'
                              % (cell.width, self.table.colcount))
-            context = ('\\par\n\\vskip-\\baselineskip\\strut\\end{varwidth}%\n') + context
+            context = ('\\par\n\\vskip-\\baselineskip'
+                       '\\vbox{\\hbox{\\strut}}\\end{varwidth}%\n') + context
             self.needs_linetrimming = 1
         if len(node) > 2 and len(node.astext().split('\n')) > 2:
             self.needs_linetrimming = 1
@@ -1805,7 +1828,7 @@ class LaTeXTranslator(nodes.NodeVisitor):
                 # TODO non vertical space for other alignments.
                 align = '\\begin{flush%s}' % node.attributes['align']
                 align_end = '\\end{flush%s}' % node.attributes['align']
-            self.body.append('\\begin{figure}[%s]%s\n' % (
+            self.body.append('\n\\begin{figure}[%s]%s\n' % (
                 self.elements['figure_align'], align))
             if any(isinstance(child, nodes.caption) for child in node):
                 self.body.append('\\capstart\n')
@@ -1985,6 +2008,8 @@ class LaTeXTranslator(nodes.NodeVisitor):
                     logger.warning('unknown index entry type %s found', type)
             except ValueError as err:
                 logger.warning(str(err))
+        if not node.get('inline', True):
+            self.body.append('\\ignorespaces ')
         raise nodes.SkipNode
 
     def visit_raw(self, node):
